@@ -117,7 +117,7 @@ parent_hwaddr_maybe_changed(NMDevice *parent, GParamSpec *pspec, gpointer user_d
          */
         s_ip6 = nm_connection_get_setting_ip6_config(connection);
         if (s_ip6)
-            nm_device_reactivate_ip6_config(device, s_ip6, s_ip6);
+            nm_device_reactivate_ip_config(device, AF_INET6, s_ip6, s_ip6);
     }
 }
 
