@@ -24,8 +24,6 @@
 typedef struct {
     GHashTable *  hash;
     GPtrArray *   array;
-    GPtrArray *   vlan_parents;
-    GHashTable *  explicit_ip_connections;
     NMConnection *bootdev_connection; /* connection for bootdev=$ifname */
     NMConnection *default_connection; /* connection not bound to any ifname */
     char *        hostname;
@@ -34,8 +32,6 @@ typedef struct {
     gboolean ignore_auto_dns;
     int      dhcp_timeout;
     char *   dhcp4_vci;
-
-    gint64 carrier_timeout_sec;
 } Reader;
 
 static Reader *
@@ -45,11 +41,8 @@ reader_new(void)
 
     reader  = g_slice_new(Reader);
     *reader = (Reader){
-        .hash = g_hash_table_new_full(nm_str_hash, g_str_equal, g_free, g_object_unref),
-        .explicit_ip_connections =
-            g_hash_table_new_full(nm_direct_hash, NULL, g_object_unref, NULL),
-        .vlan_parents = g_ptr_array_new_with_free_func(g_free),
-        .array        = g_ptr_array_new(),
+        .hash  = g_hash_table_new_full(nm_str_hash, g_str_equal, g_free, g_object_unref),
+        .array = g_ptr_array_new(),
     };
 
     return reader;
@@ -61,8 +54,6 @@ reader_destroy(Reader *reader, gboolean free_hash)
     gs_unref_hashtable GHashTable *hash = NULL;
 
     g_ptr_array_unref(reader->array);
-    g_ptr_array_unref(reader->vlan_parents);
-    g_hash_table_unref(reader->explicit_ip_connections);
     hash = g_steal_pointer(&reader->hash);
     nm_clear_g_free(&reader->hostname);
     nm_clear_g_free(&reader->dhcp4_vci);
@@ -392,7 +383,6 @@ reader_parse_ip(Reader *reader, const char *sysfs_dir, char *argument)
     NMSettingIPConfig *s_ip4 = NULL, *s_ip6 = NULL;
     gs_unref_hashtable GHashTable *ibft = NULL;
     const char *                   tmp;
-    const char *                   tmp2;
     const char *                   kind             = NULL;
     const char *                   client_ip        = NULL;
     const char *                   peer             = NULL;
@@ -421,37 +411,17 @@ reader_parse_ip(Reader *reader, const char *sysfs_dir, char *argument)
         /* ip={dhcp|on|any|dhcp6|auto6|ibft} */
         kind = tmp;
     } else {
-        tmp2 = get_word(&argument, ':');
-        if (NM_IN_STRSET(tmp2,
-                         "none",
-                         "off",
-                         "dhcp",
-                         "on"
-                         "any",
-                         "dhcp6",
-                         "auto",
-                         "auto6",
-                         "ibft")) {
-            /* <ifname>:{none|off|dhcp|on|any|dhcp6|auto|auto6|ibft} */
-            iface_spec = tmp;
-            kind       = tmp2;
-        } else {
-            /* <client-IP>:[<peer>]:<gateway-IP>:<netmask>:<client_hostname>:<kind> */
-            client_ip = tmp;
-            if (client_ip) {
-                client_ip_family = get_ip_address_family(client_ip, TRUE);
-                if (client_ip_family == AF_UNSPEC) {
-                    _LOGW(LOGD_CORE, "Invalid IP address '%s'.", client_ip);
-                    return;
-                }
-            }
-
-            peer            = tmp2;
+        client_ip_family = guess_ip_address_family(tmp);
+        if (client_ip_family != AF_UNSPEC) {
+            /* <client-IP>:[<peer>]:<gateway-IP>:<netmask>:<client_hostname>: */
+            client_ip       = tmp;
+            peer            = get_word(&argument, ':');
             gateway_ip      = get_word(&argument, ':');
             netmask         = get_word(&argument, ':');
             client_hostname = get_word(&argument, ':');
             iface_spec      = get_word(&argument, ':');
-            kind            = get_word(&argument, ':');
+        } else {
+            iface_spec = tmp;
         }
 
         if (client_hostname && !nm_sd_hostname_is_valid(client_hostname, FALSE))
@@ -462,12 +432,16 @@ reader_parse_ip(Reader *reader, const char *sysfs_dir, char *argument)
             reader->hostname = g_strdup(client_hostname);
         }
 
+        /* <ifname>:{none|off|dhcp|on|any|dhcp6|auto6|ibft} */
+
+        kind = get_word(&argument, ':');
+
         tmp                = get_word(&argument, ':');
-        dns_addr_family[0] = get_ip_address_family(tmp, FALSE);
+        dns_addr_family[0] = guess_ip_address_family(tmp);
         if (dns_addr_family[0] != AF_UNSPEC) {
             dns[0]             = tmp;
             dns[1]             = get_word(&argument, ':');
-            dns_addr_family[1] = get_ip_address_family(dns[1], FALSE);
+            dns_addr_family[1] = guess_ip_address_family(dns[1]);
             if (*argument)
                 _LOGW(LOGD_CORE, "Ignoring extra: '%s'.", argument);
         } else {
@@ -487,8 +461,6 @@ reader_parse_ip(Reader *reader, const char *sysfs_dir, char *argument)
     else
         connection = reader_get_default_connection(reader);
 
-    g_hash_table_add(reader->explicit_ip_connections, g_object_ref(connection));
-
     s_ip4 = nm_connection_get_setting_ip4_config(connection);
     s_ip6 = nm_connection_get_setting_ip6_config(connection);
 
@@ -534,8 +506,9 @@ reader_parse_ip(Reader *reader, const char *sysfs_dir, char *argument)
                 _LOGW(LOGD_CORE, "Invalid address '%s': %s", client_ip, error->message);
                 g_clear_error(&error);
             }
-        } else
-            nm_assert_not_reached();
+        } else {
+            _LOGW(LOGD_CORE, "Unrecognized address: %s", client_ip);
+        }
 
         if (address) {
             switch (client_ip_family) {
@@ -558,7 +531,7 @@ reader_parse_ip(Reader *reader, const char *sysfs_dir, char *argument)
                 nm_setting_ip_config_add_address(s_ip6, address);
                 break;
             default:
-                nm_assert_not_reached();
+                _LOGW(LOGD_CORE, "Unknown address family: %s", client_ip);
                 break;
             }
             nm_ip_address_unref(address);
@@ -593,12 +566,7 @@ reader_parse_ip(Reader *reader, const char *sysfs_dir, char *argument)
                          NULL);
         }
     } else if (NM_IN_STRSET(kind, "auto6", "dhcp6")) {
-        g_object_set(s_ip6,
-                     NM_SETTING_IP_CONFIG_METHOD,
-                     NM_SETTING_IP6_CONFIG_METHOD_AUTO,
-                     NM_SETTING_IP_CONFIG_MAY_FAIL,
-                     FALSE,
-                     NULL);
+        g_object_set(s_ip4, NM_SETTING_IP_CONFIG_MAY_FAIL, FALSE, NULL);
         if (nm_setting_ip_config_get_num_addresses(s_ip4) == 0) {
             g_object_set(s_ip4,
                          NM_SETTING_IP_CONFIG_METHOD,
@@ -650,16 +618,22 @@ reader_parse_ip(Reader *reader, const char *sysfs_dir, char *argument)
         _LOGW(LOGD_CORE, "Ignoring peer: %s (not implemented)\n", peer);
 
     if (gateway_ip && *gateway_ip) {
-        switch (get_ip_address_family(gateway_ip, FALSE)) {
-        case AF_INET:
-            g_object_set(s_ip4, NM_SETTING_IP_CONFIG_GATEWAY, gateway_ip, NULL);
-            break;
-        case AF_INET6:
-            g_object_set(s_ip6, NM_SETTING_IP_CONFIG_GATEWAY, gateway_ip, NULL);
-            break;
-        default:
+        int addr_family = guess_ip_address_family(gateway_ip);
+
+        if (nm_utils_ipaddr_is_valid(addr_family, gateway_ip)) {
+            switch (addr_family) {
+            case AF_INET:
+                g_object_set(s_ip4, NM_SETTING_IP_CONFIG_GATEWAY, gateway_ip, NULL);
+                break;
+            case AF_INET6:
+                g_object_set(s_ip6, NM_SETTING_IP_CONFIG_GATEWAY, gateway_ip, NULL);
+                break;
+            default:
+                _LOGW(LOGD_CORE, "Unknown address family: %s", gateway_ip);
+                break;
+            }
+        } else {
             _LOGW(LOGD_CORE, "Invalid gateway: %s", gateway_ip);
-            break;
         }
     }
 
@@ -708,8 +682,6 @@ reader_parse_master(Reader *reader, char *argument, const char *type_name, const
     char *               slaves;
     const char *         slave;
     char *               opts;
-    char *               opt;
-    const char *         opt_name;
     const char *         mtu = NULL;
 
     master = get_word(&argument, ':');
@@ -731,17 +703,27 @@ reader_parse_master(Reader *reader, char *argument, const char *type_name, const
 
         opts = get_word(&argument, ':');
         while (opts && *opts) {
+            gs_free_error GError *error = NULL;
+            char *                opt;
+            const char *          opt_name;
+
             opt      = get_word(&opts, ',');
             opt_name = get_word(&opt, '=');
+
+            if (!_nm_setting_bond_validate_option(opt_name, opt, &error)) {
+                _LOGW(LOGD_CORE,
+                      "Ignoring invalid bond option: %s%s%s = %s%s%s: %s",
+                      NM_PRINT_FMT_QUOTE_STRING(opt_name),
+                      NM_PRINT_FMT_QUOTE_STRING(opt),
+                      error->message);
+                continue;
+            }
             nm_setting_bond_add_option(s_bond, opt_name, opt);
         }
 
         mtu = get_word(&argument, ':');
     }
 
-    if (mtu)
-        connection_set(connection, NM_SETTING_WIRED_SETTING_NAME, NM_SETTING_WIRED_MTU, mtu);
-
     do {
         slave = get_word(&slaves, ',');
         if (slave == NULL)
@@ -755,6 +737,8 @@ reader_parse_master(Reader *reader, char *argument, const char *type_name, const
                      NM_SETTING_CONNECTION_MASTER,
                      master,
                      NULL);
+        if (mtu)
+            connection_set(connection, NM_SETTING_WIRED_SETTING_NAME, NM_SETTING_WIRED_MTU, mtu);
     } while (slaves && *slaves != '\0');
 
     if (argument && *argument)
@@ -869,9 +853,6 @@ reader_parse_vlan(Reader *reader, char *argument)
 
     if (argument && *argument)
         _LOGW(LOGD_CORE, "Ignoring extra: '%s'.", argument);
-
-    if (!nm_strv_ptrarray_contains(reader->vlan_parents, phy))
-        g_ptr_array_add(reader->vlan_parents, g_strdup(phy));
 }
 
 static void
@@ -892,11 +873,6 @@ reader_parse_rd_znet(Reader *reader, char *argument, gboolean net_ifnames)
     subchannels[0] = get_word(&argument, ',');
     subchannels[1] = get_word(&argument, ',');
 
-    /* Without subchannels we can't univocally match
-     * a device. */
-    if (!subchannels[0] || !subchannels[1])
-        return;
-
     if (nm_streq0(nettype, "ctc")) {
         if (net_ifnames == TRUE) {
             prefix = "sl";
@@ -976,7 +952,7 @@ reader_add_nameservers(Reader *reader, GPtrArray *nameservers)
 
     for (i = 0; i < nameservers->len; i++) {
         ns          = nameservers->pdata[i];
-        addr_family = get_ip_address_family(ns, FALSE);
+        addr_family = guess_ip_address_family(ns);
         if (addr_family == AF_UNSPEC) {
             _LOGW(LOGD_CORE, "Unknown address family: %s", ns);
             continue;
@@ -1032,10 +1008,7 @@ connection_set_needed_cb(gpointer key, gpointer value, gpointer user_data)
 }
 
 GHashTable *
-nmi_cmdline_reader_parse(const char *       sysfs_dir,
-                         const char *const *argv,
-                         char **            hostname,
-                         gint64 *           carrier_timeout_sec)
+nmi_cmdline_reader_parse(const char *sysfs_dir, const char *const *argv, char **hostname)
 {
     Reader *          reader;
     const char *      tag;
@@ -1069,9 +1042,6 @@ nmi_cmdline_reader_parse(const char *       sysfs_dir,
         } else if (nm_streq(tag, "rd.net.dhcp.vendor-class")) {
             if (nm_utils_validate_dhcp4_vendor_class_id(argument, NULL))
                 nm_utils_strdup_reset(&reader->dhcp4_vci, argument);
-        } else if (nm_streq(tag, "rd.net.timeout.carrier")) {
-            reader->carrier_timeout_sec =
-                _nm_utils_ascii_str_to_int64(argument, 10, 0, G_MAXINT32, 0);
         }
     }
 
@@ -1126,54 +1096,20 @@ nmi_cmdline_reader_parse(const char *       sysfs_dir,
         }
     }
 
-    for (i = 0; i < reader->vlan_parents->len; i++) {
-        NMConnection *     connection;
-        NMSettingIPConfig *s_ip;
-
-        /* Disable IP configuration for parent connections of VLANs,
-         * unless those interfaces were explicitly configured otherwise. */
-
-        connection = reader_get_connection(reader, reader->vlan_parents->pdata[i], NULL, TRUE);
-        if (!g_hash_table_contains(reader->explicit_ip_connections, connection)) {
-            s_ip = nm_connection_get_setting_ip4_config(connection);
-            if (s_ip) {
-                g_object_set(s_ip,
-                             NM_SETTING_IP_CONFIG_METHOD,
-                             NM_SETTING_IP4_CONFIG_METHOD_DISABLED,
-                             NULL);
-            }
-
-            s_ip = nm_connection_get_setting_ip6_config(connection);
-            if (s_ip) {
-                g_object_set(s_ip,
-                             NM_SETTING_IP_CONFIG_METHOD,
-                             NM_SETTING_IP6_CONFIG_METHOD_DISABLED,
-                             NULL);
-            }
-        }
-    }
-
     if (ignore_bootif)
         nm_clear_g_free(&bootif_val);
     if (bootif_val) {
         NMConnection *  connection;
         NMSettingWired *s_wired;
         const char *    bootif = bootif_val;
-        char            prefix[4];
 
-        if (!nm_utils_hwaddr_valid(bootif, ETH_ALEN)) {
-            strncpy(prefix, bootif, 3);
-            prefix[3] = '\0';
-
-            if (NM_IN_STRSET(prefix, "01-", "01:", "00-", "00:")
-                && nm_utils_hwaddr_valid(&bootif[3], ETH_ALEN)) {
-                /*
-                 * BOOTIF MAC address can be prefixed with a hardware type identifier.
-                 * "01" stays for "wired", "00" is also accepted as it means "undefined".
-                 * No others are known.
-                 */
-                bootif += 3;
-            }
+        if (!nm_utils_hwaddr_valid(bootif, ETH_ALEN) && g_str_has_prefix(bootif, "01-")
+            && nm_utils_hwaddr_valid(&bootif[3], ETH_ALEN)) {
+            /*
+             * BOOTIF MAC address can be prefixed with a hardware type identifier.
+             * "01" stays for "wired", no other are known.
+             */
+            bootif += 3;
         }
 
         connection = reader_get_connection(reader, NULL, NM_SETTING_WIRED_SETTING_NAME, FALSE);
@@ -1232,7 +1168,5 @@ nmi_cmdline_reader_parse(const char *       sysfs_dir,
 
     NM_SET_OUT(hostname, g_steal_pointer(&reader->hostname));
 
-    NM_SET_OUT(carrier_timeout_sec, reader->carrier_timeout_sec);
-
     return reader_destroy(reader, FALSE);
 }
