@@ -125,6 +125,19 @@ _l3cfg_weak_notify(gpointer data, GObject *where_the_object_was)
         g_object_unref(self);
 }
 
+NML3Cfg *
+nm_netns_get_l3cfg(NMNetns *self, int ifindex)
+{
+    NMNetnsPrivate *priv;
+
+    g_return_val_if_fail(NM_IS_NETNS(self), NULL);
+    g_return_val_if_fail(ifindex > 0, NULL);
+
+    priv = NM_NETNS_GET_PRIVATE(self);
+
+    return g_hash_table_lookup(priv->l3cfgs, &ifindex);
+}
+
 NML3Cfg *
 nm_netns_access_l3cfg(NMNetns *self, int ifindex)
 {
@@ -453,7 +466,7 @@ dispose(GObject *object)
         g_signal_handlers_disconnect_by_data(priv->platform, &priv->_self_signal_user_data);
 
     g_clear_object(&priv->platform);
-    g_clear_pointer(&priv->l3cfgs, g_hash_table_unref);
+    nm_clear_pointer(&priv->l3cfgs, g_hash_table_unref);
 
     nm_clear_pointer(&priv->rules_manager, nmp_rules_manager_unref);
 
